# -*- coding: utf-8 -*-
"""
This module provide utilities to find ephemeral port ranges for the current OS.
See http://www.ncftp.com/ncftpd/doc/misc/ephemeral_ports.html for more info
about ephemeral port ranges.

Currently only Linux and BSD (including OS X) are supported.
"""
from __future__ import absolute_import, with_statement
import subprocess

DEFAULT_EPHEMERAL_PORT_RANGE = (32768, 65535)


def port_ranges():
    """
    Returns a list of ephemeral port ranges for current machine.
    """
    try:
        return _linux_ranges()
    except (OSError, IOError):  # not linux, try BSD
        try:
            ranges = _bsd_ranges()
            if ranges:
                return ranges
        except (OSError, IOError):
            pass

    # fallback
    return [DEFAULT_EPHEMERAL_PORT_RANGE]


def _linux_ranges():
    with open("/proc/sys/net/ipv4/ip_local_port_range") as f:
        # use readline() instead of read() for linux + musl
        low, high = f.readline().split()
        return [(int(low), int(high))]


def _bsd_ranges():
    pp = subprocess.Popen(
        ["sysctl", "net.inet.ip.portrange"], stdout=subprocess.PIPE
    )
    stdout, stderr = pp.communicate()
    lines = stdout.decode("ascii").split("\n")
    out = dict(
        [
            [x.strip().rsplit(".")[-1] for x in line.split(":")]
            for line in lines
            if line
        ]
    )

    ranges = [
        # FreeBSD & Mac
        ("first", "last"),
        ("lowfirst", "lowlast"),
        ("hifirst", "hilast"),
        # OpenBSD
        ("portfirst", "portlast"),
        ("porthifirst", "porthilast"),
    ]

    res = []
    for rng in ranges:
        try:
            low, high = int(out[rng[0]]), int(out[rng[1]])
            if low <= high:
                res.append((low, high))
        except KeyError:
            pass
    return res



# -*- coding: utf-8 -*-
from __future__ import absolute_import

__version__ = "0.5.0"

from .api import (
    available_good_ports,
    available_ports,
    is_available,
    good_port_ranges,
    port_is_used,
    select_random,
    UNASSIGNED_RANGES,
)
from .store import PortStore
from .exceptions import PortForException



#!/usr/bin/env python
from setuptools import setup


setup(
    name="port-for",
    version="0.5.0",
    author="Mikhail Korobov",
    author_email="kmike84@gmail.com",
    packages=["port_for"],
    scripts=["scripts/port-for"],
    url="https://github.com/kmike/port-for/",
    license="MIT license",
    description="""Utility that helps with local TCP ports managment.
    It can find an unused TCP localhost port and remember the association.""",
    long_description=open("README.rst").read(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: POSIX",
        "Topic :: System :: Installation/Setup",
        "Topic :: System :: Systems Administration",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
    ],
    python_requires=">=3.6",
)
